@@ -10,6 +10,12 @@ sub new
 {
     my ( $class , %params ) = @_;
     my $self = bless \%params , $class;
+
+    if ($self->{lang} and not ref($self->{lang}) eq 'ARRAY')
+    {
+        $self->{lang} = [$self->{lang}];
+    }
+
     return $self;
 }
 
@@ -69,36 +75,160 @@ sub _fix_internal_tz
 sub _do_math
 {
     my ( $self , $plug , $date ) = @_;
-    my %strings = $plug->math_strings;
-    my $ago = $plug->ago;
 
-    if ( $date =~ m{$ago}mix )
+    my %relative_strings = $plug->relative;
+    my %day_strings = $plug->days;
+    my %month_strings = $plug->months;
+
+    my $instructions = {
+        ago  => {direction => 'past', units => 1},
+        from => {direction => 'future', units => 1},
+        last => {direction => 'past'},
+        next => {direction => 'future'},
+    };
+
+    foreach my $keyword (keys %relative_strings)
     {
-        my $base_dt = DateTime::Format::Flexible->base->clone;
-        if ( my ( $amount , $unit ) = $date =~ m{(\d+)\s+([^\s]+)}mx )
-        {
+        my $rx = $relative_strings{$keyword};
 
-            printf( "#    %s => %s\n", $amount, $unit ) if $ENV{DFF_DEBUG};
-            if ( exists( $strings{$unit} ) )
+        next if not (exists $instructions->{$keyword});
+
+        my $has_units = $instructions->{$keyword}->{units};
+        my $direction = $instructions->{$keyword}->{direction};
+
+        if ( $date =~ m{$rx}mix )
+        {
+            $date =~ s{$rx}{}mx;
+            if ($has_units)
             {
-                printf( "#    found: %s\n", $strings{$unit} ) if $ENV{DFF_DEBUG};
-                my $ret = $base_dt->subtract( $strings{$unit} => $amount );
-                $date =~ s{\s{0,}$amount\s+$unit\s{0,}}{}mx;
-                printf( "#    after removing amount,unit: [%s]\n", $date ) if $ENV{DFF_DEBUG};
-                $date =~ s{$ago}{}mx;
-                printf( "#    after removing ago: [%s]\n", $date ) if $ENV{DFF_DEBUG};
-                if ( $date ) # we still have more to parse...
-                {
-                    $date = $ret->ymd . ' ' . $date;
-                }
-                else
-                {
-                    $date = $ret->datetime;
-                }
-                return $date;
+                $date = $self->_set_units( $plug , $date, $direction );
             }
+            else
+            {
+                foreach my $day (keys %day_strings)
+                {
+                    if ($date =~ m{$day}mix)
+                    {
+                        $date = $self->_set_day( $plug , $date , $day , $direction );
+                        $date =~ s{$day}{}mx;
+                    }
+                }
+                foreach my $month (keys %month_strings)
+                {
+                    if ($date =~ m{$month}mix)
+                    {
+                        $date = $self->_set_month( $plug , $date , $month , $direction );
+                        $date =~ s{$month}{}mx;
+                    }
+                }
+            }
+            printf("#  after removing rx (%s): [%s]\n", $rx, $date) if $ENV{DFF_DEBUG};
+
+            $date =~ s{$keyword}{}mx;
+            $date =~ s{\s+}{ }gm;
+            $date =~ s{\s+\z}{}gm;
+            printf("#  after removing keyword (%s): [%s]\n", $keyword, $date) if $ENV{DFF_DEBUG};
+        }
+
+    }
+
+    return $date;
+}
+
+sub _set_units
+{
+    my ( $self , $plug , $date , $direction ) = @_;
+
+    my %strings = $plug->math_strings;
+    if ( my ( $amount , $unit ) = $date =~ m{(\d+)\s+([^\s]+)}mx )
+    {
+        printf( "#  %s => %s\n", $amount, $unit ) if $ENV{DFF_DEBUG};
+        if ( exists( $strings{$unit} ) )
+        {
+            my $base_dt = DateTime::Format::Flexible->base->clone;
+
+            if ( $direction eq 'past' )
+            {
+                $base_dt->subtract( $strings{$unit} => $amount );
+            }
+            if ( $direction eq 'future' )
+            {
+                $base_dt->add( $strings{$unit} => $amount );
+            }
+            $date =~ s{\s{0,}$amount\s+$unit\s{0,}}{}mx;
+
+            if ($ENV{DFF_DEBUG})
+            {
+                printf("#  found: %s\n", $strings{$unit}) ;
+                printf("#  after removing amount, unit: [%s]\n", $date);
+            }
+
+            $date = $base_dt->datetime . ' ' . $date;
         }
     }
+
+    return $date;
+}
+
+sub _set_day
+{
+    my ( $self , $plug , $date , $day , $direction ) = @_;
+
+    my %day_strings = $plug->days;
+
+    my $base_dt = DateTime::Format::Flexible->base->clone;
+    my $dow = $base_dt->day_of_week;
+    my $date_dow = $day_strings{$day};
+    if ( $direction eq 'past' )
+    {
+        my $amount = $dow - $date_dow;
+        if ($amount < 1) {$amount = 7 + $amount}
+        printf("#    subtracting %s days\n", $amount) if $ENV{DFF_DEBUG};
+
+        my $ret = $base_dt->subtract( 'days' => $amount )->truncate( to => 'day' );
+        $date = $ret->datetime . ' ' . $date;
+
+    }
+    if ( $direction eq 'future' )
+    {
+        my $amount = $date_dow - $dow;
+        if ($amount < 1) {$amount = 7 + $amount}
+        printf("#    adding %s days\n", $amount) if $ENV{DFF_DEBUG};
+
+        my $ret = $base_dt->add( 'days' => $amount )->truncate( to => 'day' );
+        $date = $ret->datetime . ' ' . $date;
+    }
+
+
+    return $date;
+}
+
+sub _set_month
+{
+    my ( $self , $plug , $date , $month , $direction ) = @_;
+
+    my %month_strings = $plug->months;
+
+    my $base_dt = DateTime::Format::Flexible->base->clone;
+    my $mon = $base_dt->month;
+    my $date_mon = $month_strings{$month};
+
+    printf("#    setting month to: %s\n", $date_mon) if $ENV{DFF_DEBUG};
+
+    $base_dt->set_month($date_mon);
+    if ($direction eq 'past' and $date_mon >= $mon)
+    {
+        $base_dt->set_year($base_dt->year - 1);
+    }
+    if ($direction eq 'future' and $date_mon <= $mon)
+    {
+        $base_dt->set_year($base_dt->year + 1);
+    }
+    $base_dt->truncate( to => 'month' );
+    printf("#    set year to: %s\n", $base_dt->year) if $ENV{DFF_DEBUG};
+
+    $date = $base_dt->datetime . ' ' . $date;
+
     return $date;
 }
 
@@ -207,7 +337,8 @@ sub _fix_hours
         if ( $date =~ m{$hour}mxi )
         {
             my $realtime = $hours{ $hour };
-            $date =~ s{$hour}{$realtime}gmix;
+            $date =~ s{T[^\s]+}{};
+            $date =~ s{$hour}{${realtime}}gmix;
             return $date;
         }
     }
@@ -227,6 +358,9 @@ sub _remove_strings
             $date =~ s{$rs}{ }gmix;
         }
     }
+    $date =~ s{\A\s+}{};
+    $date =~ s{\s+\z}{};
+
     return $date;
 }
 
